package com.fightingchaos.VASSAL.counters;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;

import VASSAL.build.GameModule;
import VASSAL.build.module.Map;
import VASSAL.build.module.documentation.HelpFile;
import VASSAL.build.module.map.MassKeyCommand;
import VASSAL.command.AddPiece;
import VASSAL.command.ChangeTracker;
import VASSAL.command.RemovePiece;
import VASSAL.counters.GamePiece;
import VASSAL.counters.Properties;
import VASSAL.counters.Stack;
import VASSAL.tools.ErrorDialog;

/**
 * Class for grouping game pieces on a map, which will keep their relative position
 * to the group and can be moved as one piece. Select pieces anywhere on the Map with 
 * GroupStackMetrics and you wil be able to move them around as one piece. see also @see UngroupCommand
 *
 */
public class GroupCommand extends MassKeyCommand {

	public void apply(Map m) {		
		if(m == null)
			return;
		
		if(!(m.getStackMetrics() instanceof GroupStackMetrics)) {
			ErrorDialog.show(
				      null, null,
				      "Grouping not supported",
				      "This map does not support grouping of pieces.\n",
				      "The command will not be executed."
				    );			
			return;
		}
		
		GamePiece[] p = m.getAllPieces();
	
		if(p.length <= 1)
			return;
		
		ArrayList<GamePiece> selected = new ArrayList<GamePiece>();
		//collect all selected
	    for(int i=0; i<p.length;i++) {
	    	if(p[i] instanceof Group) {
	    		selected.addAll(getSelected((Group)p[i]));
	    	}
	    }
	    if(selected.size() > 0) {
	    	HashMap<Stack, Stack> grps = new HashMap<Stack, Stack>();
	    	//create new group
	    	Group grp = new Group();
	    	ChangeTracker t = new ChangeTracker(grp);
	    	grp.setMap(m);
	    	//position group on map
	    	grp.setPosition(selected.get(0).getPosition());
	    	//add to new group
	    	for (int i=0; i<selected.size(); i++) {
	    		GamePiece s = selected.get(i);
	    		if(s.getParent() != null)
	    			grps.put(s.getParent(), s.getParent());
	    		grp.add(s);
			}
	    	
	    	//add group to game   	
	    	m.addPiece(grp);
	    	GameModule.getGameModule().getGameState().addPiece(grp);
	    	//remove old groups if empty
	    	for(Stack item : grps.values()) {
	    		if(item.getPieceCount() <= 0) {
	    			m.removePiece(item);
	    			GameModule.getGameModule().getGameState().removePiece(item.getId());
	    			GameModule.getGameModule().sendAndLog(new RemovePiece(item));
	    		}
	    	}
	    	m.repaint();
	    	//changes added
	    	GameModule.getGameModule().sendAndLog(new AddPiece(grp));//t.getChangeCommand());
	    	super.apply(m);
	    }
	}

	protected ArrayList<GamePiece> getSelected(Group grp) {
		ArrayList<GamePiece> l = new ArrayList<GamePiece>();
		GamePiece p = null;
		for(Iterator<GamePiece> it = grp.getPiecesIterator();it.hasNext();) {
		p=it.next();
	      if(Boolean.TRUE.equals(p.getProperty(Properties.SELECTED))) {
	    	l.add(p);
	      }
		}
	    return l;
	}
	
	  public HelpFile getHelpFile() {
		    return HelpFile.getReferenceManualPage("com.fightingchaos.VASSAL/index.html", "GroupCommand");
	  }

}
