package com.fightingchaos.VASSAL.counters;

import java.awt.AlphaComposite;
import java.awt.Color;
import java.awt.Component;
import java.awt.Composite;
import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.RenderingHints;
import java.awt.event.MouseEvent;
import java.awt.geom.AffineTransform;
import java.awt.geom.Arc2D;
import java.awt.geom.Area;
import java.awt.geom.Point2D;
import java.awt.image.BufferedImage;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.text.DecimalFormat;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Enumeration;
import java.util.List;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.KeyStroke;
import javax.swing.plaf.basic.BasicHTML;

import VASSAL.build.GameModule;
import VASSAL.build.module.GlobalOptions;
import VASSAL.build.module.Map;
import VASSAL.build.module.documentation.HelpFile;
import VASSAL.build.module.map.MovementReporter;
import VASSAL.command.ChangeTracker;
import VASSAL.command.Command;
import VASSAL.command.MoveTracker;
import VASSAL.command.NullCommand;
import VASSAL.configure.BooleanConfigurer;
import VASSAL.configure.HotKeyConfigurer;
import VASSAL.configure.IntConfigurer;
import VASSAL.configure.StringConfigurer;
import VASSAL.counters.Decorator;
import VASSAL.counters.FreeRotator;
import VASSAL.counters.GamePiece;
import VASSAL.counters.KeyCommand;
import VASSAL.counters.PieceEditor;
import VASSAL.counters.Properties;
import VASSAL.counters.Stack;
import VASSAL.i18n.PieceI18nData;
import VASSAL.tools.HashCode;
import VASSAL.tools.SequenceEncoder;
import VASSAL.tools.image.ImageUtils;
import VASSAL.tools.imageop.AbstractTileOpImpl;
import VASSAL.tools.imageop.Op;
import VASSAL.tools.imageop.RotateScaleOp;
import VASSAL.tools.imageop.ScaledImagePainter;

/**
 * A Decorator that rotates a group of GamePieces to an arbitrary angle
 * based on FreeRotator.
 * known issues: 
 * -problems with maps which are scaled down or up
 * -cloning will create pieces on a spot (like on a stack)
 */
public class GroupRotator extends FreeRotator {
	
	  public static final String ID = "group_rotate;";
	  private static final int VERSION = 2;

	  public static final int INACTIVE = 0;
	  public static final int SELECT_PIVOT = 1;
	  public static final int SELECT_SIDE = 2;
	  public static final int ROTATE = 3;

	  protected KeyCommand rotateFreeCommand;
	  protected KeyCommand[] commands;
	  protected KeyStroke rotateFreeKey;

	  protected String rotateFreeText = "Rotate around free pivot";
	  protected String name = "Group Rotate";
	  
	  protected String INCH = "Inch";
	  protected double scale = 45.0;
	  
	  private int rotate_state = INACTIVE;
	  private Point pivotPoint = null;
	  private Point wheelPoint = null;
	  private double distance = 0.0;
	  
	  protected boolean inInteractiveRotate = false;
	  
	  ScaledImagePainter imagePainter;
	  private static Font font = new Font("Dialog", Font.PLAIN, 10);
	  private static Color color = new Color(1,1,1);
	  private static Color area_color = new Color(0.2f,0.2f,0.2f,0.5f);

	  private Point mousePos;

	  private boolean rotateClockwise;
	  
	  //FIXME grr copy & paste from Labeler.java better make it an own public class
	  protected class LabelOp extends AbstractTileOpImpl {
	    private final String txt;
	    private final Font font;
	    private final Color fg;
	    private final Color bg;
	    private final int hash;
	    
	    public LabelOp(String txt, Font font, Color fg, Color bg) {
	      this.txt = txt;
	      this.font = font;
	      this.fg = fg;
	      this.bg = bg;
	      hash = HashCode.hash(txt) ^
	             HashCode.hash(font) ^
	             HashCode.hash(fg) ^
	             HashCode.hash(bg);
	    }

	    public List<VASSAL.tools.opcache.Op<?>> getSources() {
	      return Collections.emptyList();
	    }

	    public BufferedImage eval() throws Exception {
	      // determine whether we are HTML and fix our size
	      final JLabel label = buildDimensions();

	      // draw nothing if our size is zero
	      if (size.width <= 0 || size.height <= 0) return ImageUtils.NULL_IMAGE;
	 
	      // prepare the target image
	      final BufferedImage im = ImageUtils.createCompatibleImage(
	        size.width,
	        size.height,
	        bg == null || bg.getTransparency() != Color.OPAQUE
	      );

	      final Graphics2D g = im.createGraphics();
	      g.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
	                         RenderingHints.VALUE_ANTIALIAS_ON);
	      
	      // paint the background  
	      if (bg != null) {
	        g.setColor(bg);
	        g.fillRect(0, 0, size.width, size.height);
	      }

	      // paint the foreground
	      if (fg != null) {
	        if (label != null) {
	          label.paint(g);
	        }
	        else {
	          g.setColor(fg);
	          g.setFont(font);
	  
	          final FontMetrics fm = g.getFontMetrics(font);
	          g.drawString(txt, 0, size.height - fm.getDescent());
	        }
	      }

	      g.dispose();
	      return im;
	    }

	    private JLabel buildDimensions() {
	      // Build and return a JLabel if we need to render HTML,
	      // then determine the dimensions of the label.
	      final JLabel label;

	      if (BasicHTML.isHTMLString(txt)) {
	        label = new JLabel(txt);
	        label.setForeground(fg);
	        label.setFont(font);
	        label.setSize(label.getPreferredSize());
	        size = label.getSize();
	      }
	      else {
	        label = null;

	        final Graphics2D g = ImageUtils.NULL_IMAGE.createGraphics();
	        final FontMetrics fm = g.getFontMetrics(font);
	        size = new Dimension(fm.stringWidth(txt), fm.getHeight());
	        g.dispose();
	      }

	      return label; 
	    }

	    @Override
	    protected void fixSize() {
	      if ((size = getSizeFromCache()) == null) {
	        buildDimensions();
	      
	        // ensure that our area is nonempty
	        if (size.width <= 0 || size.height <= 0) {
	          size.width = size.height = 1;
	        } 
	      }
	    }

	    @Override
	    public boolean equals(Object o) {
	      if (this == o) return true;
	      if (!(o instanceof LabelOp)) return false;

	      final LabelOp lop = (LabelOp) o;
	      return (txt == null ? lop.txt == null : txt.equals(lop.txt)) &&
	             (font == null ? lop.font == null : font.equals(lop.font)) &&
	             (fg == null ? lop.fg == null : fg.equals(lop.fg)) &&
	             (bg == null ? lop.bg == null : bg.equals(lop.bg));
	    }

	    @Override
	    public int hashCode() {
	      return hash;
	    }
	  }
	  
	  private void init() {
		    tempAngle = startAngle = 0;
	        imagePainter = new ScaledImagePainter();
	        
	        try {
				scale  = Double.parseDouble(GameModule.getGameModule().getProperty(INCH).toString());
			} catch (NumberFormatException e) {
				//FIXME how do we report this? logger/chatter?
				scale = 45.0;
			}
	        scale = 2*Math.PI/scale/360.0;
	  }
	  
	  public GroupRotator() {
		    this(ID + VERSION+";1;Rotate;82,130;", null);
		    init();
	  }

	  public GroupRotator(String type, GamePiece inner) {
	    mySetType(type);
	    setInner(inner);
	    init();
	  }

//	  private double centerX() {
//	    // The center is not on a vertex for pieces with odd widths.
//	    return (piece.boundingBox().width % 2) / 2.0;
//	  }
//
//	  private double centerY() {
//	    // The center is not on a vertex for pieces with odd heights.
//	    return (piece.boundingBox().height % 2) / 2.0;
//	  }

	  public double getAngle() {
	    return useUnrotatedShape ? 0.0 : startAngle;
	  }

	  /**
	   * @param	absolute angle to be set in degree
	   */
	  public void setAngle(double angle) {		  
	   startAngle = angle%360;//%(PI2);//values between 0-2PI
	  }

	  public void mySetType(String type) {
//System.out.println("mySet:"+type);
		if(type.startsWith(FreeRotator.ID)) {
//			super.mySetType(type);
			return;
		}
	    type = type.substring(ID.length());
	    final SequenceEncoder.Decoder st = new SequenceEncoder.Decoder(type, ';');
//System.out.println("mySet:"+type);
	    String isInt = st.nextToken();
	    int version = 0;
	    rotateCWKey = null;
	    rotateCCWKey = null;
	    
	    
	    rotateCWText = "";
	    rotateCCWText = "";
	    rotateFreeText = "";
	    rotateFreeKey = null;
	    try{
	    	version = Integer.parseInt(isInt);
	    } catch(NumberFormatException e) {//old version 1 without fixed angle rotation
	    	rotateFreeText = isInt;
		    rotateFreeKey = st.nextKeyStroke(null);
		    name = st.nextToken("");
		    commands = null;
		    return;
	    }
	    if(version == VERSION){//actual version 2
		    validAngles = new double[Integer.parseInt(st.nextToken(""))];
		    for (int i = 0; i < validAngles.length; ++i) {
		      validAngles[i] = -i * (360.0 / validAngles.length);
		    }
		    if (validAngles.length == 1) {
		    	rotateFreeText = st.nextToken("");
			    rotateFreeKey = st.nextKeyStroke(null);
		    }
		    else {
		      rotateCWKey = st.nextKeyStroke(null);
		      rotateCCWKey = st.nextKeyStroke(null);
		      rotateCWText = st.nextToken("");
		      rotateCCWText = st.nextToken("");
		    }
		    name = st.nextToken("");
		    commands = null;	
	    }
	  }
	  
	  //area of the wheel arc visualization
	  protected Area getArea() {
	    Area a;
	    int size = ((int)Math.abs(pivotPoint.distance(wheelPoint.x, wheelPoint.y)));
	    double dist =tempAngle-startAngle;
	    Rectangle bbox = new Rectangle(pivotPoint.x-size,pivotPoint.y-size,
	    		2*size,2*size);
	    if(rotateClockwise) {
	    	dist = dist -180;
//	    	System.out.println("cw 1 from:"+startAngle+180+" dist:"+(dist)+" tmp:"+tempAngle);

	    	if(dist>0.0)
	    		dist = dist-360;
//	    	a = new Area(new Arc2D.Double(bbox,startAngle+180, tempAngle-startAngle-180, Arc2D.PIE));
	    	a = new Area(new Arc2D.Double(bbox,startAngle+180, dist, Arc2D.PIE));
//	    	System.out.println("cw 2 from:"+startAngle+180+" dist:"+(dist)+" tmp:"+tempAngle);
	    }else{
//	    	System.out.println("ccw 1 from:"+startAngle+" dist:"+(dist)+" tmp:"+tempAngle);
	    	if(dist<0.0)
	    		dist = 360.0+tempAngle-startAngle;
	    	a = new Area(new Arc2D.Double(bbox,startAngle, dist, Arc2D.PIE));
//	    	System.out.println("ccw 2 from:"+startAngle+" dist:"+(dist)+" tmp:"+tempAngle);

	    }

	    return a;
	  }

	  //draw the transparent wheel and the text of the units
	  //rotated and do not rotate the units
	  public void draw(final Graphics g,
	                   final int x,
	                   final int y,
	                   final Component obs,
	                   final double zoom) {
		 //draw wheel
		    if (rotate_state == ROTATE) {
		    	// The transparency is only draw on a Map.View component.  Only the GamePiece is drawn within
		    	// other windows (Counter Palette, etc.)
			  if (obs instanceof Map.View) {
			    Graphics2D g2d = (Graphics2D) g;
			
			    Color oldColor = g2d.getColor();
			    g2d.setColor(area_color);
			
			    Composite oldComposite = g2d.getComposite();
			    g2d.setComposite(AlphaComposite.getInstance(AlphaComposite.SRC_OVER, 0.5f));
			
			    Area a = getArea();
			
			    if (zoom != 1.0) {
			      a = new Area(AffineTransform.getScaleInstance(zoom,zoom).createTransformedShape(a));
			    }
			    g2d.fill(a);
			
			    g2d.setColor(oldColor);
			    g2d.setComposite(oldComposite);
			    
			    String txt = "    "+getRotationMessage();
			    imagePainter.setSource(new LabelOp(txt, font, color,null));
			    imagePainter.draw(g, mousePos.x, mousePos.y, zoom, obs);
			  }
			}

		//draw piece static with startAngle
	    if (startAngle == 0.0) {
	      piece.draw(g, x, y, obs, zoom);
	    }
	    else {
	      RotateScaleOp op;

	      if (getGpOp() != null && getGpOp().isChanged()) {
	        gpOp = Op.piece(piece);
	        rotOp.clear();
	        op = Op.rotateScale(gpOp, startAngle, zoom);        
	        rotOp.put(startAngle, op);
	      }
	      else {
	        op = rotOp.get(startAngle);
	        if (op == null || op.getScale() != zoom) {
	          op = Op.rotateScale(gpOp, startAngle, zoom);
	          rotOp.put(startAngle, op);
	        }
	      }

	      final Rectangle r = boundingBox();
	      g.drawImage(op.getImage(),
	                    x + (int) (zoom * r.x),
	                    y + (int) (zoom * r.y),
	                    obs);

	    }
	  }

	  private String getRotationMessage() {
		    DecimalFormat form = new DecimalFormat();
		    form.applyPattern("# ##0.0#");
		    //draw text of units wheeled
		    double f = tempAngle-startAngle;
		    if(rotateClockwise) {
		    	f= f-180.0;
//System.out.println("cw 1 f:"+f);
		    	if(f>0.0)
		    		f = 360.0-f;
//System.out.println("cw 2 f:"+f);
		    }
		    else{
//System.out.println("ccw 1 f:"+f);
		    	if(f<0.0)
		    		f= 360.0+tempAngle-startAngle;
//System.out.println("ccw 2 f:"+f);
		    }
//System.out.println("dis:"+distance);
		    String txt = form.format(distance*scale*(f%360));//+"  "+INCH;
		    return txt;
	}

	public void draw(Graphics g, Map map) {
		  //removed ghost drawing
	  }

	  /**
	   * Rotate pos around piv by angle 
	   * @param pos		point to be rotated
	   * @param piv		point to rotate around
	   * @param angle	angle in degrees to rotate
	   * @return		the rotated point
	   */
	  private Point getRotatedPosition(Point pos, Point piv, double angle) {
		    //prepare transform around provided pivot
		    final AffineTransform t = 
			      AffineTransform.getRotateInstance(-PI_180 *angle,
	                      piv.x,
	                      piv.y);

		    //rotate the point around the specified pivot
		    final Point2D newPos2D =
		      new Point2D.Float(pos.x, pos.y);
		    t.transform(newPos2D, newPos2D);
		    //convert to int map positions
		    return new Point((int) Math.round(newPos2D.getX()),
		                     (int) Math.round(newPos2D.getY()));
	  }
	  
	  public String myGetType() {
	    final SequenceEncoder se = new SequenceEncoder(';');
	    se.append(VERSION);
	    se.append(validAngles.length);
	    if (validAngles.length == 1) {
	    	se.append(rotateFreeText);
	    	se.append(rotateFreeKey);
	    }
	    else {
	      se.append(rotateCWKey)
	        .append(rotateCCWKey)
	        .append(rotateCWText)
	        .append(rotateCCWText);
	    }
	    se.append(name);
//System.out.println("myGet:"+ID + se.getValue());
	    return ID + se.getValue();
	  }

	  public String myGetState() {
	      return "" + startAngle;
	  }

	  public void mySetState(String state) {
	     startAngle = Double.valueOf(state).doubleValue();
	  }

	  public KeyCommand[] myGetKeyCommands() {
		    if (commands == null) {
			      final ArrayList<KeyCommand> l = new ArrayList<KeyCommand>();
			      final GamePiece outer = Decorator.getOutermost(this);
			      rotateFreeCommand = new KeyCommand(rotateFreeText, rotateFreeKey, outer);
			      rotateCWCommand = new KeyCommand(rotateCWText, rotateCWKey, outer);
			      rotateCCWCommand = new KeyCommand(rotateCCWText, rotateCCWKey, outer);

			      if (validAngles.length == 1) {
					if (rotateFreeText.length() > 0) {
					    l.add(rotateFreeCommand);
					    rotateFreeCommand.setEnabled(rotateFreeKey != null);
					}
			        rotateCWCommand.setEnabled(false);
			        rotateCCWCommand.setEnabled(false);
			      }
			      else {
			        if (rotateCWText.length() > 0 && rotateCCWText.length() > 0) {
			          l.add(rotateCWCommand);
			          l.add(rotateCCWCommand);
			        }
			        else if (rotateCWText.length() > 0) {
			          l.add(rotateCWCommand);
			          rotateCCWCommand.setEnabled(rotateCCWKey != null);
			        }
			        else if (rotateCCWText.length() > 0) {
			          l.add(rotateCCWCommand);
			          rotateCWCommand.setEnabled(rotateCWKey != null);
			        }
			        rotateFreeCommand.setEnabled(false);
			      }
			      commands = l.toArray(new KeyCommand[l.size()]);
			    }
		    	if(this.getMap() != null && (!(getMap().getStackMetrics() instanceof GroupStackMetrics))){
		    		//not enabled if not on a grouping map
		    		for(KeyCommand com : commands)
		    			com.setEnabled(false);
		    	}
	    		
			    return commands;
	  }

	  public Command myKeyEvent(KeyStroke stroke) {
	    myGetKeyCommands();
	    Command c = null;
	    if (rotateFreeCommand.matches(stroke)) {
	        if(!inInteractiveRotate)
	      	  beginInteractiveRotate();
	    } else if (rotateCWCommand.matches(stroke)) {
	      angleIndex = (angleIndex + 1) % validAngles.length;
	      performFixRotation(validAngles[angleIndex]);
	    }
	    else if (rotateCCWCommand.matches(stroke)) {
	      angleIndex = (angleIndex - 1 + validAngles.length) % validAngles.length;
	      performFixRotation(validAngles[angleIndex]);
	    }
	    return c;
	  }

	  private void performFixRotation(double angle) {
		// set pivotPoint, rotateClockwise or ccw, tempangle
		// and perform the rotation
		if(inInteractiveRotate)
			return;
		
		Stack p = getParent();
		if (p != null && p instanceof Group) {
			Rectangle rect = p.getShape().getBounds();
			this.pivotPoint = new Point(p.getPosition());
			this.pivotPoint.translate((int)rect.getCenterX(), (int)rect.getCenterY());
			if(angle>0.0)
				this.rotateClockwise = true;
			else
				this.rotateClockwise = false;
			
			this.tempAngle = angle;
			
			performRotation();
		}
		
	}

	public void beginInteractiveRotate() {
		  if(getParent() != null)
			  if(!getInnermost(this).equals(getInnermost(getParent().topPiece())))
					  return;
			  
//System.out.println("begin d angle:"+tempAngle+ "s:"+startAngle);
		  if(inInteractiveRotate)
			  return;
		  
		  inInteractiveRotate = true;
		  rotate_state = SELECT_PIVOT;
		  
		tempAngle = startAngle;
//System.out.println("adj d angle:"+tempAngle+ "s:"+startAngle);
	    getMap().pushMouseListener(this);
	    getMap().addDrawComponent(this);
	    getMap().getView().addMouseMotionListener(this);
	    getMap().getView()
	            .setCursor(Cursor.getPredefinedCursor(Cursor.CROSSHAIR_CURSOR));
	    pivot = getPosition();
	  }

	  public void mouseClicked(MouseEvent e) {
		  //it can happen that the models are in progress of rotation then deleted and receive
		  //focus messages while being removed from the map
		  if(getMap() == null)
			  return;
		  //first select pivot point
		  if(this.rotate_state == SELECT_PIVOT) {
			pivotPoint = getMap().mapCoordinates(e.getPoint());
		  	rotate_state = SELECT_SIDE;
//System.out.println("Pivot choosen");
		  } else if (this.rotate_state == SELECT_SIDE) {
			  //second select point to rotate and ignore further clicked
			  wheelPoint = getMap().mapCoordinates(e.getPoint());
			  mousePos = wheelPoint;
			  rotate_state = ROTATE;
//			  drawGhost = true;
			  //check if the rotation wheel should be clockwise or counterclockwise
			  //by checking if the center lies on the left, or on the right side
			  //of our wheel/pivot vector  (later used by rotationwheel)
			  Point x2 = new Point(pivotPoint.x - wheelPoint.x, pivotPoint.y - wheelPoint.y);
			  Point x3 = new Point(pivotPoint.x - getPosition().x, pivotPoint.y - getPosition().y);
			  
			  distance = pivotPoint.distance(wheelPoint);
			  
			  double spat = x2.x*x3.y-x2.y*x3.x;
			  
			  rotateClockwise = false;
				  if(spat <=0)
					  rotateClockwise = true;
//System.out.println("Wheel choosen");
		  } //else
//System.out.println("Ignored");
		  
		  //else ignore inactive or rotate
	  }

	  public void mousePressed(MouseEvent e) {
	  //removed stuff from superclass (ghost & start angle actualization)
		  //no ghost, start is fixed from last rotation
	  }

	  public void mouseReleased(MouseEvent e) {
		if (rotate_state != ROTATE)
			return;

		//only perform it with the to piece of the stack (otherwise would happen x times)
		if (getParent() != null)
			if (!getInnermost(this)
					.equals(getInnermost(getParent().topPiece())))
				return;

		try {
			if (inInteractiveRotate) {
				performRotation();
			}
		} finally {
			if (inInteractiveRotate)
				inInteractiveRotate = false;

			rotate_state = INACTIVE;
			drawGhost = false;
			getMap().getView().setCursor(null);
			getMap().removeDrawComponent(this);
			getMap().popMouseListener();
			getMap().getView().removeMouseMotionListener(this);
		}
	}

	  /**
	   * This code performs the rotation of the pieces in the group, taking the
	   * pivot point and the angle and calculating the correct angle corresponding to the
	   * position of each individual piece.
	   */
	  //FIXME eventually post messages for being rotated X distance far for whole group?
	  private void performRotation() {
			Command c = new NullCommand();
			
			// do this on all pieces and set their pivot to the corner the
			// pivot was done

			Stack p = getParent();
			GamePiece piece = null;
			GamePiece next = null;
			Point pos = null;
			Point piv3 = null;
			double angle;
			if (p != null && p instanceof Group) {
				ChangeTracker ch = new ChangeTracker(p);
				if (rotateClockwise)
					angle = ((tempAngle) - startAngle - 180) % 360;
				else
					angle = ((tempAngle) - startAngle) % 360;
				tempAngle = startAngle + angle;
//System.out.println("angle:"+angle+" temp:"+tempAngle+" start:"+startAngle);
				Enumeration<GamePiece> pieces = p.getPieces();
				while (pieces.hasMoreElements()) {
					next = pieces.nextElement();
					next = Decorator.getOutermost(next);
					piece = Decorator
							.getDecorator(next, GroupRotator.class);
					if (piece != null) {
						MoveTracker tracker = new MoveTracker(piece);
						ChangeTracker chan = new ChangeTracker(piece);
						next.setProperty(Properties.MOVED, Boolean.TRUE);
						pos = ((Group) p).getPosition(next);
						// piv3 = getRotatedPosition(pos,
						// this.pivotPoint,((tempAngle%360)-startAngle));
//System.out.println("Grp: "+p.getPosition()+" piece:"+pos+" piv:"+pivotPoint);
						piv3 = getRotatedPosition(pos, this.pivotPoint,
								angle);
//System.out.println("Grp: "+p.getPosition()+" piece:"+pos+" piv:"+piv3);
						// adjust the position of this piece(and the group
						// delta position)
						((Group) p).setPosition(next, piv3);
						// need to do this here as we want to adjust it only
						// (temp%360) - start
						// otherwise pass this as the parameter if done
						// before as on set the start is reseted
						// ((GroupRotator)piece).setAngle(tempAngle);
						((GroupRotator) piece).setAngle(tempAngle);
						c = c.append(tracker.getMoveCommand()).append(chan.getChangeCommand());
					}
				}
				((Group) p).recalculateCenter();
				c.append(ch.getChangeCommand());
			} else {
				ChangeTracker tracker = new ChangeTracker(p);
				setAngle(tempAngle);
				c = tracker.getChangeCommand();
			}
			
			//test from translate
		      MovementReporter r = new MovementReporter(c);
		      if (GlobalOptions.getInstance().autoReportEnabled()) {        
		        Command reportCommand = r.getReportCommand();
		        if (reportCommand != null) {
		          reportCommand.execute();
		        }
		        c.append(reportCommand);
		      }
		      c.append(r.markMovedPieces());
		    //end test
			GameModule.getGameModule().sendAndLog(c);	}

	public void mouseDragged(MouseEvent e) {
		  mousePos = getMap().mapCoordinates(e.getPoint());
		final double myAngle = getRelativeAngle(mousePos, pivot);
	      tempAngle = myAngle;
	    getMap().repaint();
	  }

	  /**
	   * returns the angle between 0 and 360 degrees
	   */
	  private double getRelativeAngle(Point p, Point origin) {
	    double myAngle;
	    double dx = p.x -origin.x;
	    double dy = p.y -origin.y;
	    
	    if(dy == 0) {
	    	myAngle = Math.PI/2.0;
	    } else {
	    	myAngle = Math.atan2(dy,dx);
	    }
	    myAngle = myAngle*180/Math.PI;	    
	    if(dy<0)
	    	myAngle = -myAngle;
	   else
		   myAngle = 360 -myAngle;

	    return myAngle;
	  }

	  public void mouseMoved(MouseEvent e) {
		  if(rotate_state == ROTATE)
			  mousePos = getMap().mapCoordinates(e.getPoint());
	  }

	  public String getDescription() {
	    return "Group Rotate";
	  }

	  public PieceEditor getEditor() {
	    return new Ed(this);
	  }
	  
	  public PieceI18nData getI18nData() {
	    return getI18nData(new String[] {rotateFreeText}, 
	                       new String[] {getCommandDescription(name, "Wheel pieces command")});
	  }

	  private static class Ed implements PieceEditor, PropertyChangeListener {
	    private HotKeyConfigurer rndKeyConfig;
	    private StringConfigurer nameConfig;
	    private JTextField rndCommand;
	    private Box rndControls;
	    private IntConfigurer facingsConfig;
	    private BooleanConfigurer anyConfig;
	    private HotKeyConfigurer cwKeyConfig;
	    private HotKeyConfigurer ccwKeyConfig;
	    private JTextField cwCommand;
	    private JTextField ccwCommand;
	    private Box cwControls;
	    private Box ccwControls;

	    private JPanel panel;

	    public Ed(GroupRotator p) {
	      nameConfig = new StringConfigurer(null, "Description:  ", p.name);
	      rndKeyConfig = new HotKeyConfigurer(null, "Command for free rotation:  ", p.rotateFreeKey);
	      facingsConfig = new IntConfigurer(null, "Number of allowed facings:  ", new Integer(p.validAngles.length == 1 ? 6 : p.validAngles.length));
	      anyConfig = new BooleanConfigurer(null, "Allow arbitrary rotations",
	  	        Boolean.valueOf(p.validAngles.length == 1));
	      cwKeyConfig = new HotKeyConfigurer(null, "Command to rotate clockwise:  ", p.rotateCWKey);
	      ccwKeyConfig = new HotKeyConfigurer(null, "Command to rotate counterclockwise:  ", p.rotateCCWKey);

	      
	      panel = new JPanel();
	      panel.setLayout(new BoxLayout(panel, BoxLayout.Y_AXIS));

	      rndControls = Box.createHorizontalBox();
	      panel.add(nameConfig.getControls());
	      panel.add(anyConfig.getControls());
	      rndControls.add(rndKeyConfig.getControls());
	      rndControls.add(new JLabel(" Menu text:  "));
	      rndCommand = new JTextField(12);
	      rndCommand.setMaximumSize(rndCommand.getPreferredSize());
	      rndCommand.setText(p.rotateFreeText);
	      rndControls.add(rndCommand);
	      panel.add(rndControls);
	      panel.add(facingsConfig.getControls());
	      cwControls = Box.createHorizontalBox();
	      cwControls.add(cwKeyConfig.getControls());
	      cwControls.add(new JLabel(" Menu text:  "));
	      cwCommand = new JTextField(12);
	      cwCommand.setMaximumSize(cwCommand.getPreferredSize());
	      cwCommand.setText(p.rotateCWText);
	      cwControls.add(cwCommand);
	      panel.add(cwControls);

	      ccwControls = Box.createHorizontalBox();
	      ccwControls.add(ccwKeyConfig.getControls());
	      ccwControls.add(new JLabel(" Menu text:  "));
	      ccwCommand = new JTextField(12);
	      ccwCommand.setMaximumSize(ccwCommand.getPreferredSize());
	      ccwCommand.setText(p.rotateCCWText);
	      ccwControls.add(ccwCommand);
	      panel.add(ccwControls);

	      anyConfig.addPropertyChangeListener(this);
	      propertyChange(null);
	    }

	    public void propertyChange(PropertyChangeEvent evt) {
		  final boolean any = Boolean.TRUE.equals(anyConfig.getValue());
	      rndControls.setVisible(any);
	      facingsConfig.getControls().setVisible(!any);
	      cwControls.setVisible(!any);
	      ccwControls.setVisible(!any);
	      panel.revalidate();
	    }

	    public Component getControls() {
	      return panel;
	    }

	    public String getType() {
	      final SequenceEncoder se = new SequenceEncoder(';');
	      se.append(VERSION);
	      if (Boolean.TRUE.equals(anyConfig.getValue())) {
		        se.append("1")
		          .append((KeyStroke) rndKeyConfig.getValue())
		          .append(rndCommand.getText() == null
		                  ? "" : rndCommand.getText().trim());
		      }
		      else {
		        se.append(facingsConfig.getValueString())
		          .append((KeyStroke) cwKeyConfig.getValue())
		          .append((KeyStroke) ccwKeyConfig.getValue())
		          .append(cwCommand.getText() == null
		                  ? "" : cwCommand.getText().trim())
		          .append(ccwCommand.getText() == null
		                  ? "" : ccwCommand.getText().trim());
		      }
		      se.append(nameConfig.getValueString());		      
		      return ID + se.getValue();
	    }

	    public String getState() {
	      return "0";
	    }
	  }
	  
	  public HelpFile getHelpFile() {
		    return HelpFile.getReferenceManualPage("com.fightingchaos.VASSAL/GroupRotator.html", "GroupRotator");
	  }
}