package com.fightingchaos.VASSAL.counters;

import java.awt.Point;
import java.awt.Rectangle;
import java.awt.Shape;
import java.awt.geom.AffineTransform;

import VASSAL.build.module.map.StackMetrics;
import VASSAL.counters.GamePiece;
import VASSAL.counters.Properties;
import VASSAL.counters.Stack;


/**
 * GroupStackMetrics just displays the pieces as they are as we are only
 * interrested in grouping them, and not doing anything fancy with them
 * @author tmi
 *
 */
public class GroupStackMetrics extends StackMetrics {

// just for testing purpose
//	  public void draw(Stack stack, Point location, Graphics g, Map map, double zoom, Rectangle visibleRect) {
//System.out.println("GroupStackMetrics draw P @"+location.x+"/"+location.y);
//		super.draw(stack, location, g, map, zoom, visibleRect);
//	  }
//	  
//	  public void draw(Stack stack, Graphics g, int x, int y, Component obs, double zoom) {
//System.out.println("GroupStackMetrics draw @"+x+"/"+y);
//		super.draw(stack, g, x, y, obs, zoom);
//	  }
	
	/**
	   * Fill the argument arrays with the positions, selection bounds and bounding boxes of the pieces in the argument stack
	   * @param parent The parent Stack
	   * @param positions If non-null will contain a {@link Point} giving the position of each piece in <code>parent</code>
	   * @param shapes If non-null will contain a {@link Shape} giving the shape of for each piece in <code>parent</code>
	   * @param boundingBoxes If non-null will contain a {@link Rectangle} giving the bounding box for each piece in <code>parent</code>
	   * @param x the x-location of the parent
	   * @param y the y-location of the parent
	   * @return the number of pieces processed in the stack
	   */
	@Override
	  public int getContents(Stack parent, Point[] positions, Shape[] shapes, Rectangle[] boundingBoxes, int x, int y) {
		if(!(parent instanceof Group)) 
			return super.getContents(parent, positions, shapes, boundingBoxes, x, y);
		
	    int count = parent.getMaximumVisiblePieceCount();
	    Point orgPos = null;
	    
	    if (positions != null) {
	      count = Math.min(count, positions.length);
	    }
	    if (boundingBoxes != null) {
	      count = Math.min(count, boundingBoxes.length);
	    }
	    if (shapes != null) {
	      count = Math.min(count,shapes.length);
	    }

	    for (int index = 0; index < count; ++index) {
	      GamePiece child = parent.getPieceAt(index);
	      if (Boolean.TRUE.equals(child.getProperty(Properties.INVISIBLE_TO_ME))) {
	        Rectangle blank = new Rectangle(x, y, 0, 0);
	        if (positions != null) {
	          positions[index] = blank.getLocation();
	        }
	        if (boundingBoxes != null) {
	          boundingBoxes[index] = blank;
	        }
	        if (shapes != null) {
	          shapes[index] = blank;
	        }
	      }
	      else {
//	    	  if(parent instanceof Group) {
	    		  orgPos = ((Group)parent).getRelativePosition(child);
	    		  orgPos.translate(x, y);
//	    	  }
//	    	  else
//	    		  orgPos = parent.getPosition();
	    	  
	    	  
	          if (positions != null) {
	              positions[index] = orgPos;
	            }
	            if (boundingBoxes != null) {
	              child.setProperty(Properties.USE_UNROTATED_SHAPE,Boolean.TRUE);
	              Rectangle bbox = child.boundingBox();
	              child.setProperty(Properties.USE_UNROTATED_SHAPE,Boolean.FALSE);
	              bbox.translate(orgPos.x, orgPos.y);
	              boundingBoxes[index] = bbox;
	            }
	            if (shapes != null) {
	              Shape s = child.getShape();
	              s = AffineTransform.getTranslateInstance(orgPos.x,orgPos.y).createTransformedShape(s);
	              shapes[index] = s;
	            }
	      }
	    }
	    return count;
	  }
	  
	  @Override
	  public Stack createStack(GamePiece p) {
		    if (isStackingEnabled()) {
		      Group s = new Group(p);
		      return s;
		    }
		    else {
		      return null;
		    }
		  }
}
