package com.fightingchaos.VASSAL.counters;

import java.util.ArrayList;
import java.util.HashMap;

import VASSAL.build.GameModule;
import VASSAL.build.module.Map;
import VASSAL.build.module.documentation.HelpFile;
import VASSAL.command.AddPiece;
import VASSAL.command.RemovePiece;
import VASSAL.counters.GamePiece;
import VASSAL.counters.Stack;
import VASSAL.tools.ErrorDialog;

import com.fightingchaos.VASSAL.counters.Group;
import com.fightingchaos.VASSAL.counters.GroupCommand;
import com.fightingchaos.VASSAL.counters.GroupStackMetrics;


/**
 * Class for un-grouping before grouped game pieces on a map, which will retain their relative position
 * when reset to a single game piece. see also @see GroupCommand
 *
 */
public class UngroupCommand extends GroupCommand {

	//FIXME had the case where ungroup did just nothing, debug needed
	public void apply(Map m) {
		if(m == null)
			return;
		
		if(!(m.getStackMetrics() instanceof GroupStackMetrics)) {
			ErrorDialog.show(
				      null, null,
				      "Grouping not supported",
				      "This map does not support ungrouping of pieces.\n",
				      "The command will not be executed."
				    );				
			return;
		}
		
		GamePiece[] p = m.getAllPieces();
	
		//FIXME VERIFY:a single piece is already singled out, not a group with 5 selected or so ?
		if(p.length <= 1)
			return;
		
		ArrayList<GamePiece> selected = new ArrayList<GamePiece>();
		//collect all selected
	    for(int i=0; i<p.length;i++) {
	    	if(p[i] instanceof Group) {
	    		selected.addAll(getSelected((Group)p[i]));
	    	}
	    }
	    if(selected.size() > 0) { 
	    	HashMap<Stack, Stack> grps = new HashMap<Stack, Stack>();
	    	for (int i=0; i<selected.size(); i++) {
	    		//get sure to collect the pieces themself, not groups again
	    		GamePiece s = selected.get(i);
	    		//add parent if present
	    		if(s.getParent() != null)
	    			grps.put(s.getParent(), s.getParent());
	    		//create new group
		    	Stack grp = m.getStackMetrics().createStack(null);
		    	grp.setMap(m);
		    	//position group on map
		    	if(s.getParent() != null && s.getParent() instanceof Group)
		    		grp.setPosition(((Group)s.getParent()).getPosition(s));
		    	else
		    		grp.setPosition(s.getPosition());
		    	//add to new group
	    		grp.add(s);
	    		//add group to game
		    	m.addPiece(grp);
		    	GameModule.getGameModule().getGameState().addPiece(grp);
		    	GameModule.getGameModule().sendAndLog(new AddPiece(grp));

			}
	    	//remove old groups if empty
	    	for(Stack item : grps.values()) {
	    		if(item.getPieceCount() <= 0) {
	    			m.removePiece(item);
	    			GameModule.getGameModule().getGameState().removePiece(item.getId());
	    	    	GameModule.getGameModule().sendAndLog(new RemovePiece(item));
	    		}
	    	}
	    	m.repaint();
	    }
	}
	
	  public HelpFile getHelpFile() {
		    return HelpFile.getReferenceManualPage("com.fightingchaos.VASSAL/index.html", "UngroupCommand");
	  }
}
